from cbapi.errors import ApiError
from cbapi.psc.base_query import QueryBuilder, PSCQueryBase
from cbapi.psc.base_query import QueryBuilderSupportMixin, IterableQueryMixin
import logging
from six import string_types

log = logging.getLogger(__name__)


class RunQuery(PSCQueryBase):
    """
    Represents a query that either creates or retrieves the
    status of a LiveQuery run.
    """

    def __init__(self, doc_class, cb):
        super().__init__(doc_class, cb)
        self._query_token = None
        self._query_body = {"device_filter": {}}
        self._device_filter = self._query_body["device_filter"]

    def device_ids(self, device_ids):
        """
        Restricts the devices that this LiveQuery run is performed on
        to the given IDs.

        :param device_ids: list of ints
        :return: This instance
        """
        if not all(isinstance(device_id, int) for device_id in device_ids):
            raise ApiError("One or more invalid device IDs")
        self._device_filter["device_ids"] = device_ids
        return self

    def device_types(self, device_types):
        """
        Restricts the devices that this LiveQuery run is performed on
        to the given device types.

        :param device_types: list of strs
        :return: This instance
        """
        if not all(isinstance(device_type, str) for device_type in device_types):
            raise ApiError("One or more invalid device types")
        self._device_filter["device_types"] = device_types
        return self

    def policy_ids(self, policy_ids):
        """
        Restricts this LiveQuery run to the given policy IDs.

        :param policy_ids: list of ints
        :return: This instance
        """
        if not all(isinstance(policy_id, int) for policy_id in policy_ids):
            raise ApiError("One or more invalid policy IDs")
        self._device_filter["policy_ids"] = policy_ids
        return self

    def where(self, sql):
        """
        Sets this LiveQuery run's underlying SQL.

        :param sql: The SQL to execute
        :return: This instance
        """
        self._query_body["sql"] = sql
        return self

    def name(self, name):
        """
        Sets this LiveQuery run's name. If no name is explicitly set,
        the run is named after its SQL.

        :param name: The run name
        :return: This instance
        """
        self._query_body["name"] = name
        return self

    def notify_on_finish(self):
        """
        Sets the notify-on-finish flag on this LiveQuery run.

        :return: This instance
        """
        self._query_body["notify_on_finish"] = True
        return self

    def submit(self):
        """
        Submits this LiveQuery run.

        :return: A new ``Run`` instance containing the run's status
        """
        if self._query_token is not None:
            raise ApiError(
                "Query already submitted: token {0}".format(self._query_token)
            )

        if "sql" not in self._query_body:
            raise ApiError("Missing LiveQuery SQL")

        url = self._doc_class.urlobject.format(self._cb.credentials.org_key)
        resp = self._cb.post_object(url, body=self._query_body)

        return self._doc_class(self._cb, initial_data=resp.json())


class RunHistoryQuery(PSCQueryBase, QueryBuilderSupportMixin, IterableQueryMixin):
    """
    Represents a query that retrieves historic LiveQuery runs.
    """
    def __init__(self, doc_class, cb):
        super().__init__(doc_class, cb)
        self._query_builder = QueryBuilder()
        self._sort = {}

    def sort_by(self, key, direction="ASC"):
        """Sets the sorting behavior on a query's results.

        Example::

        >>> cb.select(Result).run_id(my_run).where(username="foobar").sort_by("uid")

        :param key: the key in the schema to sort by
        :param direction: the sort order, either "ASC" or "DESC"
        :rtype: :py:class:`ResultQuery`
        """
        self._sort.update({"field": key, "order": direction})
        return self

    def _build_request(self, start, rows):
        request = {"start": start}

        if self._query_builder:
            request["query"] = self._query_builder._collapse()
        if rows != 0:
            request["rows"] = rows
        if self._sort:
            request["sort"] = [self._sort]

        return request

    def _count(self):
        if self._count_valid:
            return self._total_results

        url = self._doc_class.urlobject_history.format(
            self._cb.credentials.org_key
        )
        request = self._build_request(start=0, rows=0)
        resp = self._cb.post_object(url, body=request)
        result = resp.json()

        self._total_results = result["num_found"]
        self._count_valid = True

        return self._total_results

    def _perform_query(self, start=0, rows=0):
        url = self._doc_class.urlobject_history.format(
            self._cb.credentials.org_key
        )
        current = start
        numrows = 0
        still_querying = True
        while still_querying:
            request = self._build_request(start, rows)
            resp = self._cb.post_object(url, body=request)
            result = resp.json()

            self._total_results = result["num_found"]
            self._count_valid = True

            results = result.get("results", [])
            for item in results:
                yield self._doc_class(self._cb, item)
                current += 1
                numrows += 1

                if rows and numrows == rows:
                    still_querying = False
                    break

            start = current
            if current >= self._total_results:
                still_querying = False
                break


class ResultQuery(PSCQueryBase, QueryBuilderSupportMixin, IterableQueryMixin):
    """
    Represents a query that retrieves results from a LiveQuery run.
    """
    def __init__(self, doc_class, cb):
        super().__init__(doc_class, cb)
        self._query_builder = QueryBuilder()
        self._criteria = {}
        self._sort = {}
        self._batch_size = 100
        self._run_id = None

    def criteria(self, **kwargs):
        """Sets the filter criteria on a query's results.

        Example::

        >>> cb.select(Result).run_id(my_run).criteria(device_id=[123, 456])

        """
        self._criteria.update(kwargs)
        return self

    def sort_by(self, key, direction="ASC"):
        """Sets the sorting behavior on a query's results.

        Example::

        >>> cb.select(Result).run_id(my_run).where(username="foobar").sort_by("uid")

        :param key: the key in the schema to sort by
        :param direction: the sort order, either "ASC" or "DESC"
        :rtype: :py:class:`ResultQuery`
        """
        self._sort.update({"field": key, "order": direction})
        return self

    def run_id(self, run_id):
        """Sets the run ID to query results for.

        Example::

        >>> cb.select(Result).run_id(my_run)
        """
        self._run_id = run_id
        return self

    def _build_request(self, start, rows):
        request = {"start": start, "query": self._query_builder._collapse()}

        if rows != 0:
            request["rows"] = rows
        if self._criteria:
            request["criteria"] = self._criteria
        if self._sort:
            request["sort"] = [self._sort]

        return request

    def _count(self):
        if self._count_valid:
            return self._total_results

        if self._run_id is None:
            raise ApiError("Can't retrieve count without a run ID")

        url = self._doc_class.urlobject.format(
            self._cb.credentials.org_key, self._run_id
        )
        request = self._build_request(start=0, rows=0)
        resp = self._cb.post_object(url, body=request)
        result = resp.json()

        self._total_results = result["num_found"]
        self._count_valid = True

        return self._total_results

    def _perform_query(self, start=0, rows=0):
        if self._run_id is None:
            raise ApiError("Can't retrieve results without a run ID")

        url = self._doc_class.urlobject.format(
            self._cb.credentials.org_key, self._run_id
        )
        current = start
        numrows = 0
        still_querying = True
        while still_querying:
            request = self._build_request(start, rows)
            resp = self._cb.post_object(url, body=request)
            result = resp.json()

            self._total_results = result["num_found"]
            self._count_valid = True

            results = result.get("results", [])
            for item in results:
                yield self._doc_class(self._cb, item)
                current += 1
                numrows += 1

                if rows and numrows == rows:
                    still_querying = False
                    break

            start = current
            if current >= self._total_results:
                still_querying = False
                break


class FacetQuery(PSCQueryBase, QueryBuilderSupportMixin, IterableQueryMixin):
    """
    Represents a query that receives facet information from a LiveQuery run.
    """
    def __init__(self, doc_class, cb):
        super().__init__(doc_class, cb)
        self._query_builder = QueryBuilder()
        self._facet_fields = []
        self._criteria = {}
        self._run_id = None

    def facet_field(self, field):
        """Sets the facet fields to be received by this query.

        Example::

        >>> cb.select(ResultFacet).run_id(my_run).facet_field(["device.policy_name", "device.os"])

        :param field: Field(s) to be received, either single string or list of strings
        :return: Query object
        :rtype: :py:class:`Query`
        """
        if isinstance(field, string_types):
            self._facet_fields.append(field)
        else:
            for name in field:
                self._facet_fields.append(name)
        return self

    def criteria(self, **kwargs):
        """Sets the filter criteria on a query's results.

        Example::

        >>> cb.select(ResultFacet).run_id(my_run).criteria(device_id=[123, 456])

        """
        self._criteria.update(kwargs)
        return self

    def run_id(self, run_id):
        """Sets the run ID to query results for.

        Example::

        >>> cb.select(ResultFacet).run_id(my_run)
        """
        self._run_id = run_id
        return self

    def _build_request(self, rows):
        terms = {"fields": self._facet_fields}
        if rows != 0:
            terms["rows"] = rows
        request = {"query": self._query_builder._collapse(), "terms": terms}
        if self._criteria:
            request["criteria"] = self._criteria
        return request

    def _perform_query(self, rows=0):
        if self._run_id is None:
            raise ApiError("Can't retrieve results without a run ID")

        url = self._doc_class.urlobject.format(
            self._cb.credentials.org_key, self._run_id
        )
        request = self._build_request(rows)
        resp = self._cb.post_object(url, body=request)
        result = resp.json()
        results = result.get("terms", [])
        for item in results:
            yield self._doc_class(self._cb, item)



#!/usr/bin/env python

import sys
from cbapi.example_helpers import build_cli_parser, get_cb_response_object


def output_info(server, info):
    print(server)
    print("-" * 80)
    for key in sorted(info.keys()):
        print("%-30s : %s" % (key, info[key]))


def main():
    parser = build_cli_parser()
    args = parser.parse_args()
    cb = get_cb_response_object(args)

    output_info(cb.url, cb.info())


if __name__ == "__main__":
    sys.exit(main())



# This file is based on stucchio's py-lru-cache module, original copy at
# https://github.com/stucchio/Python-LRU-cache
# licensed under MIT

try:
    from collections import OrderedDict
except ImportError:
    # Python 2.6 compatibility
    from ordereddict import OrderedDict

from cbapi.six import iteritems
from itertools import islice
import time
import threading
import weakref


def lru_cache_function(max_size=1024, expiration=15*60):
    """
    >>> @lru_cache_function(3, 1)
    ... def f(x):
    ...    print "Calling f(" + str(x) + ")"
    ...    return x
    >>> f(3)
    Calling f(3)
    3
    >>> f(3)
    3
    """
    def wrapper(func):
        return LRUCachedFunction(func, LRUCacheDict(max_size, expiration))
    return wrapper


def _lock_decorator(func):
    """
    If the LRUCacheDict is concurrent, then we should lock in order to avoid
    conflicts with threading, or the ThreadTrigger.
    """
    def withlock(self, *args, **kwargs):
        if self.concurrent:
            with self._rlock:
                return func(self, *args, **kwargs)
        else:
            return func(self, *args, **kwargs)
    withlock.__name__ == func.__name__
    return withlock


class LRUCacheDict(object):
    """ A dictionary-like object, supporting LRU caching semantics.

    >>> d = LRUCacheDict(max_size=3, expiration=3)
    >>> d['foo'] = 'bar'
    >>> d['foo']
    'bar'
    >>> import time
    >>> time.sleep(4) # 4 seconds > 3 second cache expiry of d
    >>> d['foo']
    Traceback (most recent call last):
        ...
    KeyError: 'foo'
    >>> d['a'] = 'A'
    >>> d['b'] = 'B'
    >>> d['c'] = 'C'
    >>> d['d'] = 'D'
    >>> d['a'] # Should return value error, since we exceeded the max cache size
    Traceback (most recent call last):
        ...
    KeyError: 'a'

    By default, this cache will only expire items whenever you poke it - all methods on
    this class will result in a cleanup. If the thread_clear option is specified, a background
    thread will clean it up every thread_clear_min_check seconds.

    If this class must be used in a multithreaded environment, the option concurrent should be
    set to true. Note that the cache will always be concurrent if a background cleanup thread
    is used.
    """
    def __init__(self, max_size=1024, expiration=15*60, thread_clear=False, thread_clear_min_check=60, concurrent=True):
        self.max_size = max_size
        self.expiration = expiration

        self.__values = {}
        self.__expire_times = OrderedDict()
        self.__access_times = OrderedDict()
        self.thread_clear = thread_clear
        self.concurrent = concurrent or thread_clear
        if self.concurrent:
            self._rlock = threading.RLock()
        if thread_clear:
            t = self.EmptyCacheThread(self)
            t.start()

    class EmptyCacheThread(threading.Thread):
        daemon = True

        def __init__(self, cache, peek_duration=60):
            me = self

            def kill_self(o):
                me

            self.ref = weakref.ref(cache)
            self.peek_duration = peek_duration
            super(LRUCacheDict.EmptyCacheThread, self).__init__()

        def run(self):
            while self.ref():
                c = self.ref()
                if c:
                    next_expire = c.cleanup()
                    if (next_expire is None):
                        time.sleep(self.peek_duration)
                    else:
                        time.sleep(next_expire+1)
                c = None

    @_lock_decorator
    def size(self):
        return len(self.__values)

    @_lock_decorator
    def clear(self):
        """
        Clears the dict.

        >>> d = LRUCacheDict(max_size=3, expiration=1)
        >>> d['foo'] = 'bar'
        >>> d['foo']
        'bar'
        >>> d.clear()
        >>> d['foo']
        Traceback (most recent call last):
        ...
        KeyError: 'foo'
        """
        self.__values.clear()
        self.__expire_times.clear()
        self.__access_times.clear()

    def __contains__(self, key):
        return key in self

    @_lock_decorator
    def has_key(self, key):
        """
        This method should almost NEVER be used. The reason is that between the time
        has_key is called, and the key is accessed, the key might vanish.

        You should ALWAYS use a try: ... except KeyError: ... block.

        >>> d = LRUCacheDict(max_size=3, expiration=1)
        >>> d['foo'] = 'bar'
        >>> d['foo']
        'bar'
        >>> import time
        >>> if d.has_key('foo'):
        ...    time.sleep(2) #Oops, the key 'foo' is gone!
        ...    d['foo']
        Traceback (most recent call last):
        ...
        KeyError: 'foo'
        """
        return key in self.__values

    @_lock_decorator
    def __setitem__(self, key, value):
        t = int(time.time())
        self.__delete__(key)
        self.__values[key] = value
        self.__access_times[key] = t
        self.__expire_times[key] = t + self.expiration
        self.cleanup()

    @_lock_decorator
    def __getitem__(self, key):
        t = int(time.time())
        del self.__access_times[key]
        self.__access_times[key] = t
        self.cleanup()
        return self.__values[key]

    @_lock_decorator
    def __delete__(self, key):
        if key in self.__values:
            del self.__values[key]
            del self.__expire_times[key]
            del self.__access_times[key]

    @_lock_decorator
    def cleanup(self):
        marked_for_deletion = set()

        if self.expiration is None:
            return None
        t = int(time.time())

        # Delete expired
        next_expire = None
        for k, v in iteritems(self.__expire_times):
            if v < t:
                marked_for_deletion.add(k)
            else:
                next_expire = v
                break

        for k in marked_for_deletion:
            self.__delete__(k)

        marked_for_deletion = set()

        # If we have more than self.max_size items, delete the oldest
        if len(self.__values) > self.max_size:
            number_to_delete = len(self.__values) - self.max_size
            marked_for_deletion = [k for k in islice(self.__access_times, number_to_delete)]
            for k in marked_for_deletion:
                self.__delete__(k)

        if not (next_expire is None):
            return next_expire - t
        else:
            return None


class LRUCachedFunction(object):
    """
    A memoized function, backed by an LRU cache.

    >>> def f(x):
    ...    print "Calling f(" + str(x) + ")"
    ...    return x
    >>> f = LRUCachedFunction(f, LRUCacheDict(max_size=3, expiration=3) )
    >>> f(3)
    Calling f(3)
    3
    >>> f(3)
    3
    >>> import time
    >>> time.sleep(4) #Cache should now be empty, since expiration time is 3.
    >>> f(3)
    Calling f(3)
    3
    >>> f(4)
    Calling f(4)
    4
    >>> f(5)
    Calling f(5)
    5
    >>> f(3) #Still in cache, so no print statement. At this point, 4 is the least recently used.
    3
    >>> f(6)
    Calling f(6)
    6
    >>> f(4) #No longer in cache - 4 is the least recently used, and there are at least 3 others
    items in cache [3,4,5,6].
    Calling f(4)
    4

    """
    def __init__(self, function, cache=None):
        if cache:
            self.cache = cache
        else:
            self.cache = LRUCacheDict()
        self.function = function
        self.__name__ = self.function.__name__

    def __call__(self, *args, **kwargs):
        key = repr((args, kwargs)) + "#" + self.__name__
        # In principle a python repr(...) should not return any # characters.
        try:
            return self.cache[key]
        except KeyError:
            value = self.function(*args, **kwargs)
            self.cache[key] = value
            return value


if __name__ == "__main__":
    import doctest
    doctest.testmod()



#!/usr/bin/env python

import sys
from cbapi.response.models import Process
from cbapi.example_helpers import build_cli_parser, get_cb_response_object


def main():
    parser = build_cli_parser("Enumerate USB Devices")
    parser.add_argument("--start", "-s", action="store", default=None, dest="start_time",
                        help="Start time (example: -2h)")

    args = parser.parse_args()
    cb = get_cb_response_object(args)

    query_string = r'regmod:registry\machine\system\currentcontrolset\control\deviceclasses\{53f56307-b6bf-11d0-94f2-00a0c91efb8b}\*'  # noqa: E501
    if args.start_time:
        query_string += ' start:{0:s}'.format(args.start_time)

    for proc in cb.select(Process).where(query_string):
        for rm in proc.regmods:
            if "{53f56307-b6bf-11d0-94f2-00a0c91efb8b}" in rm.path:
                pieces = rm.path.split("usbstor#disk&")
                if len(pieces) < 2:
                    print("WARN:::: {0}".format(str(pieces)))
                else:
                    device_info = pieces[1]  # .split('{53f56307-b6bf-11d0-94f2-00a0c91efb8b}')[0]
                    print(device_info)


if __name__ == "__main__":
    sys.exit(main())



from ..utils import convert_query_params
from ..query import PaginatedQuery
from ..errors import UnauthorizedError, ApiError
from distutils.version import LooseVersion


from cbapi.connection import BaseAPI
import logging

log = logging.getLogger(__name__)


class CbProtectionAPI(BaseAPI):
    """The main entry point into the Carbon Black Enterprise Protection API.

    :param str profile: (optional) Use the credentials in the named profile when connecting to the Carbon Black server.
        Uses the profile named 'default' when not specified.

    Usage::

    >>> from cbapi import CbEnterpriseProtectionAPI
    >>> cb = CbEnterpriseProtectionAPI(profile="production")
    """
    def __init__(self, *args, **kwargs):
        super(CbProtectionAPI, self).__init__(product_name="protection", *args, **kwargs)

        self._server_info = None
        try:
            self._populate_server_info()
        except UnauthorizedError:
            raise UnauthorizedError(uri=self.url, message="Invalid API token for server {0:s}.".format(self.url))

        log.debug('Connected to Cb server version %s at %s'
                  % (self._server_info['ParityServerVersion'], self.session.server))
        self.cb_server_version = LooseVersion(self._server_info['ParityServerVersion'])

    def _perform_query(self, cls, **kwargs):
        if hasattr(cls, "_query_implementation"):
            return cls._query_implementation(self)
        else:
            return Query(cls, self, **kwargs)

    def _populate_server_info(self):
        self._server_info = dict((sc['name'], sc['value'])
                                 for sc in self.get_object("/api/bit9platform/v1/serverConfig"))

    @property
    def info(self):
        return self._server_info

    def select(self, cls, unique_id=None, *args, **kwargs):
        """Prepares a query against the Carbon Black data store.

        :param class cls: The Model class (for example, Computer, Process, Binary, FileInstance) to query
        :param unique_id: (optional) The unique id of the object to retrieve, to retrieve a single object by ID

        :returns: An instance of the Model class if a unique_id is provided, otherwise a Query object
        """

        try:
            min_server_version = cls._minimum_server_version()
            if self.cb_server_version < min_server_version:
                raise ApiError(
                    "{0} API requires at least server version {1}".format(cls.__name__, min_server_version))
        except AttributeError:
            pass

        return super(CbProtectionAPI, self).select(cls, unique_id=unique_id, *args, **kwargs)


class CbEnterpriseProtectionAPI(CbProtectionAPI):
    """
    Backwards compatibility for previous scripts
    """
    pass


class Query(PaginatedQuery):
    """Represents a prepared query to the Carbon Black Enterprise Protection server.

    This object is returned as part of a :py:meth:`CbEnterpriseProtectionAPI.select`
    operation on models requested from the Carbon Black
    Enterprise Protection server. You should not have to create this class yourself.

    The query is not executed on the server until it's accessed, either as an iterator (where it will generate values
    on demand as they're requested) or as a list (where it will retrieve the entire result set and save to a list).
    You can also call the Python built-in ``len()`` on this object to retrieve the total number of items matching
    the query.

    The syntax for query :py:meth:where and :py:meth:sort methods can be found in the
    `Enterprise Protection API reference`_ posted on the Carbon Black Developer Network website.

    Examples::

    >>> from cbapi.protection import CbEnterpriseProtectionAPI, Computer
    >>> cb = CbEnterpriseProtectionAPI()
    >>> query = cb.select(Computer)                     # returns a Query object matching all Computers
    >>> query = query.where("ipAddress:10.201.2.*")     # add a filter to this Query
    >>> query = query.sort("processorSpeed DESC")       # sort by computer processor speed, descending
    >>> for comp in query:                              # uses the iterator to retrieve all results
    >>>     print(comp.name)
    >>> comps = query[:10]                              # retrieve the first ten results
    >>> len(query)                                      # retrieve the total count

    Notes:
        - The slicing operator only supports start and end parameters, but not step. ``[1:-1]`` is legal, but
          ``[1:2:-1]`` is not.
        - You can chain where clauses together to create AND queries; only objects that match all ``where`` clauses
          will be returned.

    .. _Enterprise Protection API reference:
        https://developer.carbonblack.com/reference/enterprise-protection/8.0/rest-api/
    """
    def __init__(self, doc_class, cb, query=None):
        super(Query, self).__init__(doc_class, cb, None)

        if query:
            self._query = [query]
        else:
            self._query = []

        self._sort_by = None
        self._group_by = None
        self._batch_size = 1000

    def _clone(self):
        nq = self.__class__(self._doc_class, self._cb)
        nq._query = self._query[::]
        nq._sort_by = self._sort_by
        nq._group_by = self._group_by
        nq._batch_size = self._batch_size
        return nq

    def where(self, q):
        """Add a filter to this query.

        :param str q: Query string - see the `Enterprise Protection API reference`_.
        :return: Query object
        :rtype: :py:class:`Query`

        .. _Enterprise Protection API reference:
            https://developer.carbonblack.com/reference/enterprise-protection/8.0/rest-api/
        """
        nq = self._clone()
        nq._query.append(q)
        return nq

    def and_(self, q):
        """Add a filter to this query. Equivalent to calling :py:meth:`where` on this object.

        :param str q: Query string - see the `Enterprise Protection API reference`_.
        :return: Query object
        :rtype: :py:class:`Query`

        .. _Enterprise Protection API reference:
            https://developer.carbonblack.com/reference/enterprise-protection/8.0/rest-api/
        """
        return self.where(q)

    def sort(self, new_sort):
        """Set the sort order for this query.

        :param str new_sort: Sort order - see the `Enterprise Protection API reference`_.
        :return: Query object
        :rtype: :py:class:`Query`

        .. _Enterprise Protection API reference:
            https://developer.carbonblack.com/reference/enterprise-protection/8.0/rest-api/
        """
        new_sort = new_sort.strip()
        nq = self._clone()
        if len(new_sort) == 0:
            nq._sort_by = None
        else:
            nq._sort_by = new_sort
        return nq

    def _count(self):
        args = {'limit': -1}
        if self._query:
            args['q'] = self._query

        query_args = convert_query_params(args)
        self._total_results = int(self._cb.get_object(self._doc_class.urlobject,
                                                      query_parameters=query_args).get("count", 0))
        self._count_valid = True
        return self._total_results

    def _search(self, start=0, rows=0):
        # iterate over total result set, 1000 at a time
        args = {}
        args['offset'] = start
        if self._sort_by:
            args['sort'] = self._sort_by
        if rows:
            args['limit'] = start + rows
        else:
            args['limit'] = start + self._batch_size

        current = start
        numrows = 0

        if self._query:
            args['q'] = self._query

        still_querying = True

        while still_querying:
            query_args = convert_query_params(args)
            result = self._cb.get_object(self._doc_class.urlobject, query_parameters=query_args)

            if len(result) == 0:
                break

            for item in result:
                yield item
                current += 1
                numrows += 1
                if rows and numrows == rows:
                    still_querying = False
                    break

            args['offset'] = current + 1

            if len(result) < self._batch_size:
                break



#!/usr/bin/env python

import sys
from cbapi.response.models import Process, Binary
from cbapi.example_helpers import build_cli_parser, get_cb_response_object
from cbapi.errors import ObjectNotFoundError


def main():
    parser = build_cli_parser("Search for cmd.exe writing to exe and dll filepaths")

    args = parser.parse_args()
    cb = get_cb_response_object(args)

    for proc in cb.select(Process).where("process_name:cmd.exe (filemod:*.exe or filemod:*.dll)"):
        for fm in proc.filemods:
            if not fm.path.lower().endswith((".exe", ".dll")):
                continue

            signed = ""
            product_name = ""

            if fm.type == "LastWrote" and fm.md5:
                try:
                    b = cb.select(Binary, fm.md5)
                    signed = b.signed
                    product_name = b.product_name
                except ObjectNotFoundError:
                    pass

            print("%s,%s,%s,%s,%s,%s,%s,%s,%s" % (str(fm.timestamp), proc.hostname, proc.username, proc.path,
                                                  fm.path, fm.type, fm.md5, signed, product_name))


if __name__ == "__main__":
    sys.exit(main())



import boto3
import json
import time
from cbapi.response.models import BannedHash
from cbapi.example_helpers import build_cli_parser, get_cb_response_object

processed_list = set()

cb = None
watchlist_name = None


def process_events(data):
    #
    # Split on newline
    #
    for event in data.split('\n'):
        if not event:
            continue

        #
        # Load the event as json
        #
        event_json = json.loads(event)

        #
        # Check for watchlist event type
        #
        if event_json.get('type', '') == "watchlist.hit.process" or \
                event_json.get('type', '') == "watchlist.hit.binary":
            #
            # Check if matches our watchlist_name
            #
            md5sum = event_json.get('docs', [])[0].get('process_md5', '')
            if event_json.get('watchlist_name', '').lower() == watchlist_name.lower():
                print("[+]: Banning Hash: {}".format(md5sum))

                try:
                    bh = cb.create(BannedHash)
                    bh.md5hash = md5sum
                    bh.text = "Auto-Blacklist from s3-watchlist-ban.py"
                    bh.save()
                    print(bh)
                except Exception as e:
                    print(e.message)


def save_progress(processed_list):
    #
    # Save our progress in a log file
    #
    with open('script_progress.log', 'wb') as hfile:
        hfile.write(json.dumps(list(processed_list)))


def listen_mode(bucket):

    print("[+]: Listen Mode")

    #
    # Init Set with all current files
    #
    current_list = set()
    for obj in bucket.objects.all():
        key = obj.key
        current_list.add(key)

    #
    # Infinite loop and we only process new files we see
    #
    while True:
        for obj in bucket.objects.all():
            key = obj.key
            if key not in current_list:
                print("[+]: New File: {}".format(key))
                #
                # We have not processed this file.
                #
                body = obj.get()['Body'].read()
                process_events(body)
                current_list.add(key)
            else:
                pass

        print("Sleeping for 1 min")
        time.sleep(60)


if __name__ == "__main__":
    print("Starting s3-watchlist-ban script...")
    #
    # Argument parsing
    #
    parser = build_cli_parser()
    parser.add_argument("-w", '--watchlist',
                        help="Watchlist Name",
                        required=True)

    parser.add_argument("-b", '--bucket',
                        help="S3 Bucket Name",
                        required=True)

    parser.add_argument("-a", '--awsprofile',
                        help="AWS Credential profile",
                        required=False)

    parser.add_argument("-l", '--listen',
                        help="Listen mode and only process new events",
                        action='store_true',
                        required=False)

    args = parser.parse_args()

    if not args.profile:
        aws_profile = "default"
    else:
        aws_profile = args.awsprofile

    #
    # Connect to S3
    #
    session = boto3.Session(profile_name=aws_profile)
    s3 = session.resource('s3')
    my_bucket = s3.Bucket(args.bucket)

    #
    # Connect to Cb Response so we can create Banned Hashes
    #
    cb = get_cb_response_object(args)

    #
    # Save watchlist name from arguments
    #
    watchlist_name = args.watchlist

    if args.listen:
        listen_mode(my_bucket)

    try:
        with open('script_progress.log', 'rb') as hfile:
            for item in json.loads(hfile.read()):
                processed_list.add(item)
    except Exception:
        print("[?]: No previous progress file found: script_progress.log")
        processed_list = set()

    #
    # List all files in S3 Bucket
    #
    for obj in reversed(list(my_bucket.objects.all())):
        key = obj.key

        #
        # Check to see if we have already processed this file
        #
        if key not in processed_list:
            print("[+]: Processing file: {}".format(key))
            #
            # We have not processed this file.
            #
            body = obj.get()['Body'].read()

            process_events(body)
            processed_list.add(key)
            save_progress(processed_list)
        else:
            print("[+]: We have already processed file: {}".format(key))

    print("[+]: saving progess to file script_progress.log")
    save_progress(processed_list)



#!/usr/bin/env python

import sys
from cbapi.example_helpers import build_cli_parser, get_cb_psc_object


def main():
    parser = build_cli_parser("Get suggestions for searching alerts")
    parser.add_argument("-q", "--query", default="", help="Query string for looking for alerts")

    args = parser.parse_args()
    cb = get_cb_psc_object(args)

    suggestions = cb.alert_search_suggestions(args.query)
    for suggestion in suggestions:
        print("Search term: '{0}'".format(suggestion["term"]))
        print("\tWeight: {0}".format(suggestion["weight"]))
        print("\tAvailable with products: {0}".format(", ".join(suggestion["required_skus_some"])))


if __name__ == "__main__":
    sys.exit(main())



# Exported public API for the Cb Enterprise Response API

from __future__ import absolute_import

from .models import (
    BannedHash, Site, ThrottleRule, Alert, Feed, Sensor, User, Watchlist, Investigation, ThreatReport, Binary, Process,
    SensorGroup, FeedAction, WatchlistAction, TaggedEvent, IngressFilter, StoragePartition, Team
)
from .rest_api import CbEnterpriseResponseAPI, CbResponseAPI



#!/usr/bin/env python

import struct
from datetime import datetime, timedelta
import dateutil.parser
import socket
import codecs


cb_datetime_format = "%Y-%m-%d %H:%M:%S.%f"
# NOTE: solr_datetime_format changed in Cb 4.1 to include microseconds
solr_datetime_format = "%Y-%m-%dT%H:%M:%S.%fZ"
sign_datetime_format = "%Y-%m-%dT%H:%M:%SZ"


"""Create a Cb 4.2-style GUID from components"""


def create_42_guid(sensor_id, proc_pid, proc_createtime):
    full_guid = codecs.encode(struct.pack('>IIQ', sensor_id, proc_pid, proc_createtime), "hex")
    return '%s-%s-%s-%s-%s' % (full_guid[:8], full_guid[8:12], full_guid[12:16],
                               full_guid[16:20], full_guid[20:])


def parse_42_guid(guid):
    guid_parts = guid.split('-')
    guid_int = codecs.decode("".join(guid_parts)[:32], "hex")
    return struct.unpack('>IIQ', guid_int)


def parse_process_guid(guid):
    sensor_id, proc_pid, proc_createtime = parse_42_guid(guid)
    return sensor_id, proc_pid, datetime(1601, 1, 1) + timedelta(microseconds=proc_createtime / 10)


def convert_to_solr(dt):
    return dt.strftime(solr_datetime_format)


# TODO: change these to use dateutil.parser (see http://labix.org/python-dateutil)
def convert_from_solr(s):
    if s == -1:
        # special case for invalid processes
        return datetime.fromtimestamp(0)

    try:
        return datetime.strptime(s, solr_datetime_format)
    except ValueError:
        # try interpreting the timestamp without the milliseconds
        return datetime.strptime(s, sign_datetime_format)


def convert_from_cb(s):
    # Use dateutil.parser to parse incoming dates; flexible on what we receive, strict on what we send.
    if s is None:
        return dateutil.parser.parse("1970-01-01T00:00:00Z")
    else:
        return dateutil.parser.parse(s)


def convert_event_time(s):
    # NOTE that this will strip incoming tzinfo data... right now we do this since incoming Cb event data is
    #  always relative to the local system clock on which it was created, and apparently sometimes the TZ data
    #  is included, and sometimes it isn't... so we normalize it by stripping off the TZ data (unfortunately)
    return convert_from_cb(s).replace(tzinfo=None)


def convert_to_cb(dt):
    return dt.strftime(cb_datetime_format)


def get_constants(prefix):
    """Create a dictionary mapping socket module constants to their names."""
    return dict((getattr(socket, n), n)
                for n in dir(socket)
                if n.startswith(prefix)
                )


protocols = get_constants("IPPROTO_")



#!/usr/bin/env python
#

import sys
import cbapi.six as six
from cbapi.response.models import Watchlist
from cbapi.example_helpers import build_cli_parser, get_cb_response_object
import logging
import json

log = logging.getLogger(__name__)


if six.PY3:
    confirm_input = input
else:
    confirm_input = raw_input  # noqa: F821


def confirm(watch_list):
    prompt = 'Import watchlist {0}? y/n: '.format(watch_list)
    while True:
        answer = confirm_input(prompt)
        if not answer:
            return True
        if answer not in ['y', 'Y', 'n', 'N']:
            print('Please enter y or n.')
            continue
        if answer in ['Y', 'y']:
            return True
        if answer in ['N', 'n']:
            return False


def import_watchlists(cb, args):
    if args.watchlists:
        watchlists_to_import = args.watchlists.split(",")
    else:
        watchlists_to_import = []

    watchlists = json.load(open(args.file, "r"))
    print("Importing {0} from {1}...".format(watchlists.get("ExportDescription", "<no description>"),
                                             watchlists.get("Author", "<no author>")))
    for watchlist in watchlists.get("Watchlists", []):
        if watchlists_to_import:
            if watchlist.get("Name", "") not in watchlists_to_import:
                continue

        if args.selective:
            if not confirm(watchlist.get("Name", "")):
                continue

        new_watchlist = cb.create(Watchlist)
        new_watchlist.name = watchlist.get("Name", "")
        new_watchlist.query = watchlist.get("SearchString", "")
        new_watchlist.index_type = watchlist.get("Type", "events")
        try:
            new_watchlist.save()
            print("-> Watchlist {0} added [id={1}]".format(new_watchlist.name, new_watchlist.id))
        except Exception as e:
            print("-> Watchlist {0} not added: {1}".format(new_watchlist.name, e.message))

    print("-> Done importing! <-")


def main():
    parser = build_cli_parser("Import watchlists from shareable JSON format")
    parser.add_argument("-f", "--file", help="Select what file watchlists are read from", required=True)
    parser.add_argument("-w", "--watchlists", help="Specific watchlist(s) to import. Can be comma separated.")
    parser.add_argument("-m", "--selective", action="store_true",
                        help="Interactively select which watchlists to import")

    args = parser.parse_args()
    cb = get_cb_response_object(args)

    return import_watchlists(cb, args)


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python

import sys

from cbapi.example_helpers import build_cli_parser, get_cb_threathunter_object
from cbapi.psc.threathunter import Process
import csv
import json


def main():
    parser = build_cli_parser("Query processes")
    parser.add_argument("-p", type=str, help="process guid", default=None)
    parser.add_argument("-f", type=str, help="output file name", default=None)
    parser.add_argument("-of", type=str, help="output file format: csv or json", default="json")

    args = parser.parse_args()
    cb = get_cb_threathunter_object(args)

    if not args.p:
        print("Error: Missing Process GUID to query the process tree with")
        sys.exit(1)

    tree = cb.select(Process).where(process_guid=args.p)[0].tree()

    for idx, child in enumerate(tree.children):
        print("Child #{}".format(idx))
        print("\tName: {}".format(child.process_name))
        print("\tNumber of children: {}".format(len(child.children)))

    if args.f is not None:
        if args.of == "json":
            with open(args.f, 'w') as outfile:
                for idx, child in enumerate(tree.children):
                    json.dump(child.original_document, outfile)
        else:
            with open(args.f, 'w') as outfile:
                csvwriter = csv.writer(outfile)
                for idx, child in enumerate(tree.children):
                    csvwriter.writerow(child.original_document)


if __name__ == "__main__":
    sys.exit(main())



# Exported public API for the Cb LiveQuery API

from __future__ import absolute_import

from .rest_api import CbLiveQueryAPI
from cbapi.psc.livequery.models import Run, RunHistory, Result, DeviceSummary
from cbapi.psc.livequery.models import ResultFacet, DeviceSummaryFacet



from cbapi.connection import BaseAPI
from cbapi.errors import ApiError, ServerError
from .cblr import LiveResponseSessionManager
import logging

log = logging.getLogger(__name__)


class CbPSCBaseAPI(BaseAPI):
    """The main entry point into the Cb PSC API.

    :param str profile: (optional) Use the credentials in the named profile when connecting to the Carbon Black server.
        Uses the profile named 'default' when not specified.

    Usage::

    >>> from cbapi import CbPSCBaseAPI
    >>> cb = CbPSCBaseAPI(profile="production")
    """
    def __init__(self, *args, **kwargs):
        super(CbPSCBaseAPI, self).__init__(product_name="psc", *args, **kwargs)
        self._lr_scheduler = None

    def _perform_query(self, cls, **kwargs):
        if hasattr(cls, "_query_implementation"):
            return cls._query_implementation(self)
        else:
            raise ApiError("All PSC models should provide _query_implementation")

    # ---- LiveOps

    @property
    def live_response(self):
        if self._lr_scheduler is None:
            self._lr_scheduler = LiveResponseSessionManager(self)
        return self._lr_scheduler

    def _request_lr_session(self, sensor_id):
        return self.live_response.request_session(sensor_id)

    # ---- Device API

    def _raw_device_action(self, request):
        """
        Invokes the API method for a device action.

        :param dict request: The request body to be passed as JSON to the API method.
        :return: The parsed JSON output from the request.
        :raises ServerError: If the API method returns an HTTP error code.
        """
        url = "/appservices/v6/orgs/{0}/device_actions".format(self.credentials.org_key)
        resp = self.post_object(url, body=request)
        if resp.status_code == 200:
            return resp.json()
        elif resp.status_code == 204:
            return None
        else:
            raise ServerError(error_code=resp.status_code, message="Device action error: {0}".format(resp.content))

    def _device_action(self, device_ids, action_type, options=None):
        """
        Executes a device action on multiple device IDs.

        :param list device_ids: The list of device IDs to execute the action on.
        :param str action_type: The action type to be performed.
        :param dict options: Options for the bulk device action.  Default None.
        """
        request = {"action_type": action_type, "device_id": device_ids}
        if options:
            request["options"] = options
        return self._raw_device_action(request)

    def _action_toggle(self, flag):
        """
        Converts a boolean flag value into a "toggle" option.

        :param boolean flag: The value to be converted.
        :return: A dict containing the appropriate "toggle" element.
        """
        if flag:
            return {"toggle": "ON"}
        else:
            return {"toggle": "OFF"}

    def device_background_scan(self, device_ids, scan):
        """
        Set the background scan option for the specified devices.

        :param list device_ids: List of IDs of devices to be set.
        :param boolean scan: True to turn background scan on, False to turn it off.
        """
        return self._device_action(device_ids, "BACKGROUND_SCAN", self._action_toggle(scan))

    def device_bypass(self, device_ids, enable):
        """
        Set the bypass option for the specified devices.

        :param list device_ids: List of IDs of devices to be set.
        :param boolean enable: True to enable bypass, False to disable it.
        """
        return self._device_action(device_ids, "BYPASS", self._action_toggle(enable))

    def device_delete_sensor(self, device_ids):
        """
        Delete the specified sensor devices.

        :param list device_ids: List of IDs of devices to be deleted.
        """
        return self._device_action(device_ids, "DELETE_SENSOR")

    def device_uninstall_sensor(self, device_ids):
        """
        Uninstall the specified sensor devices.

        :param list device_ids: List of IDs of devices to be uninstalled.
        """
        return self._device_action(device_ids, "UNINSTALL_SENSOR")

    def device_quarantine(self, device_ids, enable):
        """
        Set the quarantine option for the specified devices.

        :param list device_ids: List of IDs of devices to be set.
        :param boolean enable: True to enable quarantine, False to disable it.
        """
        return self._device_action(device_ids, "QUARANTINE", self._action_toggle(enable))

    def device_update_policy(self, device_ids, policy_id):
        """
        Set the current policy for the specified devices.

        :param list device_ids: List of IDs of devices to be changed.
        :param int policy_id: ID of the policy to set for the devices.
        """
        return self._device_action(device_ids, "UPDATE_POLICY", {"policy_id": policy_id})

    def device_update_sensor_version(self, device_ids, sensor_version):
        """
        Update the sensor version for the specified devices.

        :param list device_ids: List of IDs of devices to be changed.
        :param dict sensor_version: New version properties for the sensor.
        """
        return self._device_action(device_ids, "UPDATE_SENSOR_VERSION", {"sensor_version": sensor_version})

    # ---- Alerts API

    def alert_search_suggestions(self, query):
        """
        Returns suggestions for keys and field values that can be used in a search.

        :param query str: A search query to use.
        :return: A list of search suggestions expressed as dict objects.
        """
        query_params = {"suggest.q": query}
        url = "/appservices/v6/orgs/{0}/alerts/search_suggestions".format(self.credentials.org_key)
        output = self.get_object(url, query_params)
        return output["suggestions"]

    def _bulk_threat_update_status(self, threat_ids, status, remediation, comment):
        """
        Update the status of alerts associated with multiple threat IDs, past and future.

        :param list threat_ids: List of string threat IDs.
        :param str status: The status to set for all alerts, either "OPEN" or "DISMISSED".
        :param str remediation: The remediation state to set for all alerts.
        :param str comment: The comment to set for all alerts.
        """
        if not all(isinstance(t, str) for t in threat_ids):
            raise ApiError("One or more invalid threat ID values")
        request = {"state": status, "threat_id": threat_ids}
        if remediation is not None:
            request["remediation_state"] = remediation
        if comment is not None:
            request["comment"] = comment
        url = "/appservices/v6/orgs/{0}/threat/workflow/_criteria".format(self.credentials.org_key)
        resp = self.post_object(url, body=request)
        output = resp.json()
        return output["request_id"]

    def bulk_threat_update(self, threat_ids, remediation=None, comment=None):
        """
        Update the alert status of alerts associated with multiple threat IDs.
        The alerts will be left in an OPEN state after this request.

        :param threat_ids list: List of string threat IDs.
        :param remediation str: The remediation state to set for all alerts.
        :param comment str: The comment to set for all alerts.
        :return: The request ID, which may be used to select a WorkflowStatus object.
        """
        return self._bulk_threat_update_status(threat_ids, "OPEN", remediation, comment)

    def bulk_threat_dismiss(self, threat_ids, remediation=None, comment=None):
        """
        Dismiss the alerts associated with multiple threat IDs.
        The alerts will be left in a DISMISSED state after this request.

        :param threat_ids list: List of string threat IDs.
        :param remediation str: The remediation state to set for all alerts.
        :param comment str: The comment to set for all alerts.
        :return: The request ID, which may be used to select a WorkflowStatus object.
        """
        return self._bulk_threat_update_status(threat_ids, "DISMISSED", remediation, comment)



import sys

from cbapi.example_helpers import build_cli_parser, get_cb_livequery_object
from cbapi.psc.livequery.models import Result


def main():
    parser = build_cli_parser("Search the facets of a LiveQuery run")
    parser.add_argument("-i", "--id", type=str, required=True, help="Run ID")
    parser.add_argument(
        "--result",
        action="store_true",
        help="Run facet query on results"
    )
    parser.add_argument(
        "--device_summary",
        action="store_true",
        help="Run facet query on device summaries"
    )
    parser.add_argument(
        "-f",
        "--fields",
        nargs="+",
        type=str,
        required=False,
        help="Fields to be displayed in results",
    )

    parser.add_argument("-q", "--query", type=str, required=False, help="Search query")
    parser.add_argument(
        "--device_ids",
        nargs="+",
        type=int,
        required=False,
        help="Device IDs to filter on",
    )
    parser.add_argument(
        "--device_names",
        nargs="+",
        type=int,
        required=False,
        help="Device names to filter on",
    )
    parser.add_argument(
        "--policy_ids",
        nargs="+",
        type=int,
        required=False,
        help="Policy IDs to filter on",
    )
    parser.add_argument(
        "--policy_names",
        nargs="+",
        type=int,
        required=False,
        help="Policy names to filter on",
    )
    parser.add_argument(
        "--statuses",
        nargs="+",
        type=str,
        required=False,
        help="Statuses to filter on",
    )

    args = parser.parse_args()
    if not (args.result or args.device_summary):
        print("ERROR: One of --result or --device_summary must be specified")
        return 1
    if args.result and args.device_summary:
        print("ERROR: --result and --device_summary cannot both be specified")
        return 1

    cb = get_cb_livequery_object(args)

    results = cb.select(Result).run_id(args.id)
    result = results.first()
    if result is None:
        print("ERROR: No results.")
        return 1

    if args.result:
        facets = result.query_result_facets()
    elif args.device_summary:
        facets = result.query_device_summary_facets()
    if args.fields:
        facets.facet_field(args.fields)
    if args.query:
        facets.where(args.query)
    if args.device_ids:
        facets.criteria(device_id=args.device_ids)
    if args.device_names:
        facets.criteria(device_name=args.device_names)
    if args.policy_ids:
        facets.criteria(policy_id=args.policy_ids)
    if args.policy_names:
        facets.criteria(policy_name=args.policy_names)
    if args.statuses:
        facets.criteria(status=args.statuses)

    for facet in facets:
        print(facet)


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python

import sys
from cbapi.response.models import Alert
from cbapi.example_helpers import build_cli_parser, get_cb_response_object


def main():
    parser = build_cli_parser()
    parser.add_argument("--query", action="store", default="")

    args = parser.parse_args()

    cb = get_cb_response_object(args)

    alert_query = cb.select(Alert).where(args.query)
    for alert in alert_query:
        if 'binary' in alert.alert_type:
            print("Alert with score {0:d}: Binary {1:s} matched watchlist/report {2:s}".format(alert.report_score,
                                                                                               alert.md5,
                                                                                               alert.watchlist_name))
        else:
            print("Alert with score {0:d}: Process {1:s} matched watchlist/report {2:s}".format(alert.report_score,
                                                                                                alert.process_name,
                                                                                                alert.watchlist_name))
            print("-> visit {0:s} to view this process in the Carbon Black UI.".format(alert.process.webui_link))


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python

from __future__ import absolute_import
from ..event import registry
from ..errors import CredentialError
import pika
import platform
import os
import threading
import logging
import json
from time import sleep

from cbapi.six.moves import SimpleHTTPServer, BaseHTTPServer
from cbapi.six.moves import urllib

import ssl


log = logging.getLogger(__name__)


class FileEventSource(threading.Thread):
    def __init__(self, cb, filename):
        super(FileEventSource, self).__init__()
        self.daemon = True

        self._done = False
        self._cb = cb

        self._fp = open(filename, "rb")
        log.debug("Opened %s" % filename)

    def run(self):
        while not self._fp.closed:
            line = self._fp.readline()
            if line == "":
                # wait for new events
                sleep(1)
                continue

            try:
                msg = json.loads(line)
                routing_key = msg.get("type")
                # log.debug("Received message with routing key %s" % routing_key)
                registry.eval_callback(routing_key, line, self._cb)
            except Exception:
                pass

    def stop(self):
        self._fp.close()


class EventForwarderHTTPSource(threading.Thread):
    def __init__(self, cb, listening_address, **kwargs):
        super(EventForwarderHTTPSource, self).__init__()
        self.daemon = True

        http_parts = urllib.parse.urlparse(listening_address)
        addr_parts = http_parts.netloc.split(":")
        listening_host = addr_parts[0]
        if len(addr_parts) == 2:
            port = int(addr_parts[1])
            listening_host = addr_parts[0]
        else:
            if http_parts.scheme == "https":
                port = 443
            else:
                port = 80

        self.listening_address = listening_address
        self.httpd = BaseHTTPServer.HTTPServer((listening_host, port), SimpleHTTPServer.SimpleHTTPRequestHandler)

        if http_parts.scheme == "https":
            keyfile = kwargs.get("keyfile", None)
            certfile = kwargs.get("certfile", None)
            if not keyfile or not certfile:
                raise CredentialError("Need to specify 'keyfile' and 'certfile' for HTTPS")
            self.httpd.socket = ssl.wrap_socket(self.httpd.socket, certfile=certfile, keyfile=keyfile,
                                                server_side=True)

    def run(self):
        self.httpd.serve_forever()


# This class is based on the pika asynchronous consumer example at
# http://pika.readthedocs.io/en/0.10.0/examples/asynchronous_consumer_example.html

class RabbitMQEventSource(threading.Thread):
    def __init__(self, cb):
        super(RabbitMQEventSource, self).__init__()
        self.daemon = True

        self._closing = False
        self._connection = None
        self._channel = None
        self._closing = False
        self._consumer_tag = None
        self._auto_ack = True
        self._consuming = False

        self._cb = cb
        creds = cb.credentials

        if not creds.rabbitmq_pass:
            error_text = "RabbitMQEventSource requires credentials for the event bus. Make sure that\n" + \
                         "rabbitmq_pass, rabbitmq_user, rabbitmq_host, and rabbitmq_port are defined\n" + \
                         "in the credential file"

            if cb.credential_profile_name:
                error_text += " for profile '{0}'.".format(cb.credential_profile_name)

            raise CredentialError(error_text)

        self._url = "amqp://{0}:{1}@{2}:{3}".format(creds.rabbitmq_user, creds.rabbitmq_pass, creds.rabbitmq_host,
                                                    creds.rabbitmq_port)

        self.QUEUE = "cbapi-event-handler-{0}-{1}".format(platform.uname()[1], os.getpid())
        self.EXCHANGE = "api.events"
        self.ROUTING_KEYS = registry.event_types
        self.EXCHANGE_TYPE = "topic"

    def connect(self):
        """This method connects to RabbitMQ, returning the connection handle.
        When the connection is established, the on_connection_open method
        will be invoked by pika.

        :rtype: pika.SelectConnection

        """
        log.debug('Connecting to %s', self._url)
        return pika.SelectConnection(pika.URLParameters(self._url),
                                     self.on_connection_open,
                                     stop_ioloop_on_close=False)

    def on_connection_open(self, unused_connection):
        """This method is called by pika once the connection to RabbitMQ has
        been established. It passes the handle to the connection object in
        case we need it, but in this case, we'll just mark it unused.

        :type unused_connection: pika.SelectConnection

        """
        log.debug('Connection opened')
        self.add_on_connection_close_callback()
        self.open_channel()

    def add_on_connection_close_callback(self):
        """This method adds an on close callback that will be invoked by pika
        when RabbitMQ closes the connection to the publisher unexpectedly.

        """
        log.debug('Adding connection close callback')
        self._connection.add_on_close_callback(self.on_connection_closed)

    def on_connection_closed(self, connection, reply_code, reply_text):
        """This method is invoked by pika when the connection to RabbitMQ is
        closed unexpectedly. Since it is unexpected, we will reconnect to
        RabbitMQ if it disconnects.

        :param pika.connection.Connection connection: The closed connection obj
        :param int reply_code: The server provided reply_code if given
        :param str reply_text: The server provided reply_text if given

        """
        self._channel = None
        if self._closing:
            self._connection.ioloop.stop()
        else:
            log.warning('Connection closed, reopening in 5 seconds: (%s) %s',
                        reply_code, reply_text)
            self._connection.add_timeout(5, self.reconnect)

    def reconnect(self):
        """Will be invoked by the IOLoop timer if the connection is
        closed. See the on_connection_closed method.

        """
        # This is the old connection IOLoop instance, stop its ioloop
        self._connection.ioloop.stop()

        if not self._closing:

            # Create a new connection
            self._connection = self.connect()

            # There is now a new connection, needs a new ioloop to run
            self._connection.ioloop.start()

    def open_channel(self):
        """Open a new channel with RabbitMQ by issuing the Channel.Open RPC
        command. When RabbitMQ responds that the channel is open, the
        on_channel_open callback will be invoked by pika.

        """
        log.debug('Creating a new channel')
        self._connection.channel(on_open_callback=self.on_channel_open)

    def on_channel_open(self, channel):
        """This method is invoked by pika when the channel has been opened.
        The channel object is passed in so we can make use of it.

        Since the channel is now open, we'll declare the exchange to use.

        :param pika.channel.Channel channel: The channel object

        """
        log.debug('Channel opened')
        self._channel = channel
        self.add_on_channel_close_callback()
        self.setup_exchange(self.EXCHANGE)

    def add_on_channel_close_callback(self):
        """This method tells pika to call the on_channel_closed method if
        RabbitMQ unexpectedly closes the channel.

        """
        log.debug('Adding channel close callback')
        self._channel.add_on_close_callback(self.on_channel_closed)

    def on_channel_closed(self, channel, reply_code, reply_text):
        """Invoked by pika when RabbitMQ unexpectedly closes the channel.
        Channels are usually closed if you attempt to do something that
        violates the protocol, such as re-declare an exchange or queue with
        different parameters. In this case, we'll close the connection
        to shutdown the object.

        :param pika.channel.Channel: The closed channel
        :param int reply_code: The numeric reason the channel was closed
        :param str reply_text: The text reason the channel was closed

        """
        log.warning('Channel %i was closed: (%s) %s',
                    channel, reply_code, reply_text)
        self._connection.close()

    def setup_exchange(self, exchange_name):
        """Setup the exchange on RabbitMQ by invoking the Exchange.Declare RPC
        command. When it is complete, the on_exchange_declareok method will
        be invoked by pika.

        :param str|unicode exchange_name: The name of the exchange to declare

        """
        log.debug('Declaring exchange %s', exchange_name)
        self._channel.exchange_declare(self.on_exchange_declareok,
                                       exchange_name,
                                       self.EXCHANGE_TYPE,
                                       durable=True,
                                       auto_delete=False)

    def on_exchange_declareok(self, unused_frame):
        """Invoked by pika when RabbitMQ has finished the Exchange.Declare RPC
        command.

        :param pika.Frame.Method unused_frame: Exchange.DeclareOk response frame

        """
        log.debug('Exchange declared')
        self.setup_queue(self.QUEUE)

    def setup_queue(self, queue_name):
        """Setup the queue on RabbitMQ by invoking the Queue.Declare RPC
        command. When it is complete, the on_queue_declareok method will
        be invoked by pika.

        :param str|unicode queue_name: The name of the queue to declare.

        """
        log.debug('Declaring queue %s', queue_name)
        self._channel.queue_declare(self.on_queue_declareok, queue_name, auto_delete=True)

    def on_queue_declareok(self, method_frame):
        """Method invoked by pika when the Queue.Declare RPC call made in
        setup_queue has completed. In this method we will bind the queue
        and exchange together with the routing key by issuing the Queue.Bind
        RPC command. When this command is complete, the on_bindok method will
        be invoked by pika.

        :param pika.frame.Method method_frame: The Queue.DeclareOk frame

        """
        for routing_key in self.ROUTING_KEYS:
            log.debug('Binding %s to %s with %s',
                      self.EXCHANGE, self.QUEUE, routing_key)
            self._channel.queue_bind(self.on_bindok, self.QUEUE,
                                     self.EXCHANGE, routing_key)

    def on_bindok(self, unused_frame):
        """Invoked by pika when the Queue.Bind method has completed. At this
        point we will start consuming messages by calling start_consuming
        which will invoke the needed RPC commands to start the process.

        :param pika.frame.Method unused_frame: The Queue.BindOk response frame

        """
        log.debug('Queue bound')
        self.start_consuming()

    def start_consuming(self):
        """This method sets up the consumer by first calling
        add_on_cancel_callback so that the object is notified if RabbitMQ
        cancels the consumer. It then issues the Basic.Consume RPC command
        which returns the consumer tag that is used to uniquely identify the
        consumer with RabbitMQ. We keep the value to use it when we want to
        cancel consuming. The on_message method is passed in as a callback pika
        will invoke when a message is fully received.

        """
        if not self._consuming:
            log.debug('Issuing consumer related RPC commands')
            self.add_on_cancel_callback()
            self._consumer_tag = self._channel.basic_consume(self.on_message,
                                                             self.QUEUE, no_ack=self._auto_ack)
            self._consuming = True

    def add_on_cancel_callback(self):
        """Add a callback that will be invoked if RabbitMQ cancels the consumer
        for some reason. If RabbitMQ does cancel the consumer,
        on_consumer_cancelled will be invoked by pika.

        """
        log.debug('Adding consumer cancellation callback')
        self._channel.add_on_cancel_callback(self.on_consumer_cancelled)

    def on_consumer_cancelled(self, method_frame):
        """Invoked by pika when RabbitMQ sends a Basic.Cancel for a consumer
        receiving messages.

        :param pika.frame.Method method_frame: The Basic.Cancel frame

        """
        log.debug('Consumer was cancelled remotely, shutting down: %r',
                  method_frame)
        if self._channel:
            self._channel.close()

    def acknowledge_message(self, delivery_tag):
        """Acknowledge the message delivery from RabbitMQ by sending a
        Basic.Ack RPC method for the delivery tag.

        :param int delivery_tag: The delivery tag from the Basic.Deliver frame

        """
        log.debug('Acknowledging message %s', delivery_tag)
        self._channel.basic_ack(delivery_tag)

    def stop_consuming(self):
        """Tell RabbitMQ that you would like to stop consuming by sending the
        Basic.Cancel RPC command.

        """
        if self._channel:
            log.debug('Sending a Basic.Cancel RPC command to RabbitMQ')
            self._channel.basic_cancel(self.on_cancelok, self._consumer_tag)

    def on_cancelok(self, unused_frame):
        """This method is invoked by pika when RabbitMQ acknowledges the
        cancellation of a consumer. At this point we will close the channel.
        This will invoke the on_channel_closed method once the channel has been
        closed, which will in-turn close the connection.

        :param pika.frame.Method unused_frame: The Basic.CancelOk frame

        """
        log.debug('RabbitMQ acknowledged the cancellation of the consumer')
        self.close_channel()

    def close_channel(self):
        """Call to close the channel with RabbitMQ cleanly by issuing the
        Channel.Close RPC command.

        """
        log.debug('Closing the channel')
        self._channel.close()

    def run(self):
        """Run the example consumer by connecting to RabbitMQ and then
        starting the IOLoop to block and allow the SelectConnection to operate.

        """
        self._connection = self.connect()
        self._connection.ioloop.start()

    def stop(self):
        """Cleanly shutdown the connection to RabbitMQ by stopping the consumer
        with RabbitMQ. When RabbitMQ confirms the cancellation, on_cancelok
        will be invoked by pika, which will then closing the channel and
        connection. The IOLoop is started again because this method is invoked
        when CTRL-C is pressed raising a KeyboardInterrupt exception. This
        exception stops the IOLoop which needs to be running for pika to
        communicate with RabbitMQ. All of the commands issued prior to starting
        the IOLoop will be buffered but not processed.

        """
        log.debug('Stopping')
        self._closing = True
        self.stop_consuming()
        self._connection.ioloop.stop()
        log.debug('Stopped')

    def close_connection(self):
        """This method closes the connection to RabbitMQ."""
        log.debug('Closing connection')
        self._connection.close()

    def on_message(self, unused_channel, basic_deliver, properties, body):
        """Invoked by pika when a message is delivered from RabbitMQ. The
        channel is passed for your convenience. The basic_deliver object that
        is passed in carries the exchange, routing key, delivery tag and
        a redelivered flag for the message. The properties passed in is an
        instance of BasicProperties with the message properties and the body
        is the message that was sent.

        :param pika.channel.Channel unused_channel: The channel object
        :param pika.Spec.Basic.Deliver: basic_deliver method
        :param pika.Spec.BasicProperties: properties
        :param str|unicode body: The message body

        """
        log.debug('Received message # %s with properties %s',
                  basic_deliver.delivery_tag, properties)

        registry.eval_callback(basic_deliver.routing_key, body, self._cb)

        if not self._auto_ack:
            self.acknowledge_message(basic_deliver.delivery_tag)



#!/usr/bin/env python

from __future__ import absolute_import

from cbapi.six.moves import urllib

from distutils.version import LooseVersion
from ..connection import BaseAPI
from .models import Process, Binary, Watchlist, Investigation, Alert, ThreatReport, StoragePartition
from ..errors import UnauthorizedError, ApiError
from .cblr import LiveResponseSessionManager
from .query import Query


import logging
log = logging.getLogger(__name__)


class CbResponseAPI(BaseAPI):
    """The main entry point into the Carbon Black Enterprise Response API.
    Note that calling this will automatically connect to the Carbon Black server in order to verify
    connectivity and get the server version.

    :param str profile: (optional) Use the credentials in the named profile when connecting to the Carbon Black
        server. Uses the profile named 'default' when not specified.
    :param str url: (optional, discouraged) Instead of using a credential profile, pass URL and API token to
        the constructor.
    :param str token: (optional, discouraged) API token
    :param bool ssl_verify: (optional, discouraged) Enable or disable SSL certificate verification

    Usage::

    >>> from cbapi import CbEnterpriseResponseAPI
    >>> cb = CbEnterpriseResponseAPI(profile="production")
    """
    def __init__(self, *args, **kwargs):
        timeout = kwargs.pop("timeout", 120)   # set default timeout period to two minutes, 2x the default nginx timeout
        super(CbResponseAPI, self).__init__(product_name="response", timeout=timeout, *args, **kwargs)

        self._parsed_url = urllib.parse.urlparse(self.url)
        try:
            self.server_info = self.info()
        except UnauthorizedError:
            raise UnauthorizedError(uri=self.url, message="Invalid API token for server {0:s}.".format(self.url))

        log.debug('Connected to Cb server version %s at %s' % (self.server_info['version'], self.session.server))
        self.cb_server_version = LooseVersion(self.server_info['version'])
        if self.cb_server_version < LooseVersion('5.0'):
            raise ApiError("CbEnterpriseResponseAPI only supports Cb servers version >= 5.0.0")

        self._has_legacy_partitions = False
        if self.cb_server_version >= LooseVersion('6.0'):
            legacy_partitions = [p for p in self.select(StoragePartition) if p.info.get("isLegacy", False)]
            if legacy_partitions:
                self._has_legacy_partitions = True

        self._lr_scheduler = None

    @property
    def live_response(self):
        if self._lr_scheduler is None:
            if not self.server_info.get("cblrEnabled", False):
                raise ApiError("Cb server does not support Live Response")
            self._lr_scheduler = LiveResponseSessionManager(self)

        return self._lr_scheduler

    def info(self):
        """Retrieve basic version information from the Carbon Black Enterprise Response server.

        :return: Dictionary with information retrieved from the ``/api/info`` API route
        :rtype: dict
        """
        r = self.session.get("/api/info")
        return r.json()

    def dashboard_statistics(self):
        """Retrieve dashboard statistics from the Carbon Black Enterprise Response server.

        :return: Dictionary with information retrieved from the ``/api/v1/dashboard/statistics`` API route
        :rtype: dict
        """
        r = self.session.get("/api/v1/dashboard/statistics")
        return r.json()

    def license_request(self):
        """Retrieve license request block from the Carbon Black Enterprise Response server.

        :return: License request block
        :rtype: str
        """
        r = self.session.get("/api/license")
        return r.json().get("license_request_block", "")

    def update_license(self, license_block):
        """Upload new license to the Carbon Black Enterprise Response server.

        :param str license_block: Licence block provided by Carbon Black support
        :raises ServerError: if the license is not accepted by the Carbon Black server
        """
        r = self.post_object("/api/license", {"license": license_block})
        self.raise_unless_json(r, {"status": "success"})

    def _perform_query(self, cls, **kwargs):
        if hasattr(cls, "_query_implementation"):
            return cls._query_implementation(self)
        else:
            return Query(cls, self, **kwargs)

    def from_ui(self, uri):
        """Retrieve a Carbon Black Enterprise Response object based on URL from the Carbon Black Enterprise Response
        web user interface.

        For example, calling this function with
        ``https://server/#/analyze/00000001-0000-0554-01d1-3bc4553b8c9f/1`` as the ``uri`` argument will return a new
        :py:class: cbapi.response.models.Process class initialized with the process GUID from the URL.

        :param str uri: Web browser URL from the Cb web interface
        :return: the appropriate model object for the URL provided
        :raises ApiError: if the URL does not correspond to a recognized model object
        """
        o = urllib.parse.urlparse(uri)
        if self._parsed_url.scheme != o.scheme \
            or self._parsed_url.hostname != o.hostname \
                or self._parsed_url.port != o.port:
            raise ApiError("Invalid URL provided")

        frag = o.fragment.lstrip('/')
        parts = frag.split('/')
        if len(parts) < 2:
            raise ApiError("URL endpoint does not include a unique ID: %s" % uri)

        if frag.startswith('analyze'):
            (analyze, procid, segment) = parts[:3]
            return self.select(Process, procid, int(segment))
        elif frag.startswith('binary'):
            (binary, md5) = parts[:2]
            return self.select(Binary, md5)
        elif frag.startswith('watchlist'):
            (watchlist, watchlist_id) = parts[:2]
            return self.select(Watchlist, watchlist_id)
        elif frag.startswith('search'):
            (search, raw_query) = parts[:2]
            return Query(Process, self, raw_query=raw_query)
        elif frag.startswith('binaries'):
            (binaries, raw_query) = parts[:2]
            return Query(Binary, self, raw_query=raw_query)
        elif frag.startswith('investigation'):
            (investigation, investigation_id) = parts[:2]
            return self.select(Investigation).where("id:{0:s}".format(investigation_id)).one()
        elif frag.startswith('alerts'):
            (alerts, raw_query) = parts[:2]
            return Query(Alert, self, raw_query=raw_query)
        elif frag.startswith('threats'):
            (threats, raw_query) = parts[:2]
            return Query(ThreatReport, self, raw_query=raw_query)
        elif frag.startswith('threat-details'):
            (threatdetails, feed_id, feed_title) = parts[:3]
            return self.select(ThreatReport, "%s:%s" % (feed_id, feed_title))
        elif frag.startswith('login') or not o.fragment:
            return self
        else:
            raise ApiError("Unknown URL endpoint: %s" % uri)

    def _request_lr_session(self, sensor_id):
        return self.live_response.request_session(sensor_id)

    def create_new_partition(self):
        """Create a new Solr time partition for event storage. Available in Cb Response 6.1 and above.
        This will force roll-over current hot partition into warm partition (by renaming it to a time-stamped name)
        and create a new hot partition ("writer").

        :returns: Nothing if successful.
        :raises ApiError: if there was an error creating the new partition.
        :raises ServerError: if there was an error creating the new partition.
        """
        self.post_object("/api/v1/storage/events/new_partition", None)


class CbEnterpriseResponseAPI(CbResponseAPI):
    """
    Backwards compatibility for previous scripts
    """
    pass



from cbapi.six.moves.configparser import RawConfigParser
import os
import attrdict
import logging
import cbapi.six as six

from .errors import CredentialError

log = logging.getLogger(__name__)


default_profile = {
    "url": None,
    "token": None,
    "org_key": None,

    "ssl_verify": "True",
    "ssl_verify_hostname": "True",
    "ssl_cert_file": None,
    "ssl_force_tls_1_2": "False",

    "proxy": None,
    "ignore_system_proxy": "False",

    "rabbitmq_user": "cb",
    "rabbitmq_pass": None,
    "rabbitmq_host": "localhost",
    "rabbitmq_port": 5004
}

_boolean_states = {'1': True, 'yes': True, 'true': True, 'on': True,
                   '0': False, 'no': False, 'false': False, 'off': False}


class Credentials(attrdict.AttrDict):
    def __init__(self, *args, **kwargs):
        super(Credentials, self).__init__(default_profile)
        super(Credentials, self).__init__(*args, **kwargs)

        if not self.get("url", None):
            raise CredentialError("No URL specified")
        if not self.get("token", None):
            raise CredentialError("No API token specified")

        for k in ["ssl_verify", "ssl_verify_hostname", "ignore_system_proxy", "ssl_force_tls_1_2"]:
            x = self.get(k, default_profile.get(k, "True"))
            if isinstance(x, six.string_types) and x.lower() in _boolean_states:
                self[k] = _boolean_states[x.lower()]


class CredentialStoreFactory(object):

    # CredentialStore(product_name, credential_file=credential_file)
    @staticmethod
    def getCredentialStore(product_name, credential_file):
        if credential_file is None and os.environ.get('CBAPI_TOKEN', False) and os.environ.get('CBAPI_URL', False):
            log.debug("Using Envar credential store")
            return EnvarCredentialStore()
        else:
            log.debug("Using file credential store")
            return FileCredentialStore(**{"product_name": product_name, "credential_file": credential_file})


# A CredentialStore backed by os.environ rather than by a file on disk
class EnvarCredentialStore(object):
    def __init__(self):
        # `CBAPI_URL`, `CBAPI_TOKEN`, `CBAPI_SSL_VERIFY`
        environ = os.environ
        self.cbapi_url = environ.get('CBAPI_URL', None)
        self.cbapi_token = environ.get('CBAPI_TOKEN', None)
        self.cbapi_ssl_verify = environ.get('CBAPI_SSL_VERIFY', True)
        self.org_key = environ.get('CBAPI_ORG_KEY', None)
        self.credentials = Credentials(url=self.cbapi_url, token=self.cbapi_token, ssl_verify=self.cbapi_ssl_verify)

    def get_credentials(self, profile=None):
        return self.credentials


class FileCredentialStore(object):
    def __init__(self, product_name, **kwargs):
        if product_name not in ("response", "protection", "psc"):
            raise CredentialError("Product name {0:s} not valid".format(product_name))

        self.credential_search_path = [
            os.path.join(os.path.sep, "etc", "carbonblack", "credentials.%s" % product_name),
            os.path.join(os.path.expanduser("~"), ".carbonblack", "credentials.%s" % product_name),
            os.path.join(".", ".carbonblack", "credentials.%s" % product_name),
        ]

        if "credential_file" in kwargs:
            if isinstance(kwargs["credential_file"], six.string_types):
                self.credential_search_path = [kwargs["credential_file"]]
            elif type(kwargs["credential_file"]) is list:
                self.credential_search_path = kwargs["credential_file"]

        self.credentials = RawConfigParser(defaults=default_profile)
        self.credential_files = self.credentials.read(self.credential_search_path)

    def get_credentials(self, profile=None):
        credential_profile = profile or "default"
        if credential_profile not in self.get_profiles():
            raise CredentialError("Cannot find credential profile '%s' after searching in these files: %s." %
                                  (credential_profile, ", ".join(self.credential_search_path)))

        retval = {}
        for k, v in six.iteritems(default_profile):
            retval[k] = self.credentials.get(credential_profile, k)

        if not retval["url"] or not retval["token"]:
            raise CredentialError("Token and/or URL not available for profile %s" % credential_profile)

        return Credentials(retval)

    def get_profiles(self):
        return self.credentials.sections()



#!/usr/bin/env python
#

import sys
from cbapi.response.models import Process
from cbapi.example_helpers import build_cli_parser, get_cb_response_object
from cbapi.errors import ObjectNotFoundError
import logging

log = logging.getLogger(__name__)


class CountChildren(object):
    def __init__(self):
        self.count = 0

    def walk(self, proc, depth):
        self.count += 1


class ChildCmdlines(object):
    def __init__(self):
        self.cmdlines = []

    def walk(self, proc, depth):
        self.cmdlines.append(proc.cmdline)


def count_children(proc):
    child_counter = CountChildren()
    try:
        proc.walk_children(child_counter.walk)
    except ObjectNotFoundError:
        pass
    return [child_counter.count]


def child_cmdlines(proc):
    cmdlines = ChildCmdlines()
    try:
        proc.walk_children(cmdlines.walk)
    except ObjectNotFoundError:
        pass
    return cmdlines.cmdlines


def process_hit(cb, parent_proc_name, proc_name, value, ratio, child_behavior):
    rows = []
    q = "parent_name:{0} process_name:{1}".format(parent_proc_name, proc_name)
    for child_proc in cb.select(Process).where(q):
        row = []
        row.append(parent_proc_name)
        row.append(proc_name)
        row.append(value)
        row.append(ratio)
        row.append(q)
        row.append(child_proc.hostname)
        row.append(child_proc.username)
        row.append(child_proc.webui_link)
        row.append(child_proc.cmdline)

        row.extend(child_behavior(child_proc))

        rows.append(row)
    return rows


def main():
    parser = build_cli_parser("Term Frequency Analysis")
    parser.add_argument("-p", "--percentage", action="store", default="2", dest="percentless",
                        help="Max Percentage of Term Frequency e.g., 2 ")

    process_selection = parser.add_mutually_exclusive_group(required=True)
    process_selection.add_argument("-t", "--term", action="store", default=None, dest="procname",
                                   help="Comma separated list of parent processes to get term frequency")
    process_selection.add_argument("-f", "--termfile", action="store", default=None, dest="procnamefile",
                                   help="Text file new line separated list of parent processes to get term frequency")

    output_selection = parser.add_mutually_exclusive_group(required=False)
    output_selection.add_argument("--count", action="store_true",
                                  help="Count the child processes that match [default]")
    output_selection.add_argument("--cmdline", action="store_true",
                                  help="Output list of child process command lines")

    args = parser.parse_args()
    cb = get_cb_response_object(args)

    if args.procnamefile:
        processes = [s.strip() for s in open(args.procnamefile).readlines()]
    else:
        processes = [s.strip() for s in args.procname.split(",")]

    behavior = count_children
    if args.cmdline:
        behavior = child_cmdlines

    rows = []
    for parent_proc in processes:
        process_facets = cb.select(Process).where("parent_name:{0}".format(parent_proc)).facets("process_name")
        for term in reversed(process_facets["process_name"]):
            termratio = int(float(term['ratio']))
            if int(args.percentless) >= termratio:
                rows.extend(process_hit(cb, parent_proc, term["name"], term["value"], term["ratio"], behavior))

    for row in rows:
        print(",".join([str(x) for x in row]))


if __name__ == "__main__":
    sys.exit(main())
